@@ -5,6 +5,10 @@
 package DateTime::SpanSet;
 
 use strict;
+
+use DateTime::Set;
+use DateTime::SpanSet;
+
 use Carp;
 use Params::Validate qw( validate SCALAR BOOLEAN OBJECT CODEREF ARRAYREF );
 use Set::Infinite '0.44';
